#!/bin/perl
# CLASS, SYMBOL, NAME
$prefix = "E_" ;

## Templates

@node_NN_N = ( { "SYMBOL" => "+" , "CLASS" => "${prefix}Add" ,   "OPNAME"  => "add" } ,
	       { "SYMBOL" => "-" , "CLASS" => "${prefix}Subtract" , "OPNAME"  => "sub" } ,

	       { "SYMBOL" => "*" , "CLASS" => "${prefix}Multiply" , "OPNAME"  => "mul" } ,
	       { "SYMBOL" => "/" , "CLASS" => "${prefix}Divide" ,   "OPNAME"  => "div" } ,
	       { "SYMBOL" => "%" , "CLASS" => "${prefix}Modulus" ,   "OPNAME"  => "mod" } ,
	       ) ;

@node_BB_B = ( { "SYMBOL" => "&&" , "CLASS" => "${prefix}LogicalAnd" ,  "OPNAME"  => "and" } ,
	       { "SYMBOL" => "||" , "CLASS" => "${prefix}LogicalOr" ,   "OPNAME"  => "or" } 
	       ) ;

@node_NN_B = ( { "SYMBOL" => "==" , "CLASS" => "${prefix}Equal" ,               "OPNAME"  => "eq" } ,
	       { "SYMBOL" => "!=" , "CLASS" => "${prefix}NotEqual" ,            "OPNAME"  => "ne" } ,

	       { "SYMBOL" => "<" ,  "CLASS" => "${prefix}LessThan" ,            "OPNAME"  => "lt" } ,
	       { "SYMBOL" => "<=" , "CLASS" => "${prefix}LessThanOrEqual" ,     "OPNAME"  => "le" } ,

	       { "SYMBOL" => ">" ,  "CLASS" => "${prefix}GreaterThan" ,         "OPNAME"  => "gt" } ,
	       { "SYMBOL" => ">=" , "CLASS" => "${prefix}GreaterThanOrEqual" ,  "OPNAME"  => "ge" } ,
) ;

# General
# String eq, not equal, and also the patten matching:  =~ !~
# T_StringEqual => $prefix_StringEqual (CLASS)

%misc = ( "T_StringEqual.java" =>       { "CLASS" => "${prefix}StringEqual" } ,
	  "T_StringNotEqual.java" =>    { "CLASS" => "${prefix}StringNotEqual" } ,
 	  "T_StringMatch.java" =>       { "CLASS" => "${prefix}StringMatch" } ,
 	  "T_StringNotMatch.java" =>    { "CLASS" => "${prefix}StringNotMatch" } ,
	  "T_LogicalNot.java" =>        { "CLASS" => "${prefix}LogicalNot" } ,
	  "T_UnaryMinus.java" =>        { "CLASS" => "${prefix}UnaryMinus" } ,
	  "T_UnaryPlus.java" =>         { "CLASS" => "${prefix}UnaryPlus" } ,
	  ) ;

#$dir = "src/com/hp/hpl/jena/brql/expr" ;

$templ_NN_N = readFile("templ_NN_N.java") ;
$templ_BB_B = readFile("templ_BB_B.java") ;
$templ_NN_B = readFile("templ_NN_B.java") ;
#$templ_B    = readFile("templ_B.java") ;


subst($templ_NN_N, \@node_NN_N) ;
subst($templ_BB_B, \@node_BB_B) ;
subst($templ_NN_B, \@node_NN_B) ;


# Special
foreach $t (keys %misc)
{
    #print "Template: $t\n" ;
    my $file = readFile($t) ;
    my $java= "$misc{$t}{CLASS}.java" ;
    print "Class: $java\n" ;
    $file = substOne($java, $file, $misc{$t} ) ;
    output($java, $file) ;
    
}

sub subst
{
    my($template, $nodes) = @_ ;

    foreach my $a (@$nodes)
    {
	my $file = $template ;
	my $java= "$$a{CLASS}.java" ;
	print "Class: $java\n" ;

	$file = substOne($java, $file, $a) ;

	# Check
	if ( $file =~ m/\#(.*)\#/g )
	{
	    $sym = $1 ;
	    print STDERR "Unresolved symbol: $sym: $java\n" ;
	    next ;
	}

	output($java, $file) ;
    }
}

sub output
{
    my ($java, $file) = @_ ;
    # Output
    open(J, ">$java") || die $! ;
    binmode J ;
    #open(J,">&STDOUT") || die $! ;
    print J $file ;
    close J ;
}

sub substOne
{
    my ($class, $file, $a) = @_ ;
    foreach $k (keys %$a)
    {
	warn "$java: Empty string in substitution\n" if ( $$a{$k} eq '' ) ;
	$file =~ s/\#$k\#/$$a{$k}/g ;
    }
    return $file ;
}

sub readFile
{
    my $f = @_[0] ;
    open my $fh, "$f" or die $!;
    local $/;
    my $content = <$fh>;
    close $fh;
    # Remove any excess carriage returns
    $file =~ s/\r//g ;
    return $content ;
}
